@@ -176,6 +176,8 @@ static const struct option long_options[] = {
   { "no-intra-chroma-search",   no_argument, NULL, 0 },
   { "fast-bipred",              no_argument, NULL, 0 },
   { "no-fast-bipred",           no_argument, NULL, 0 },
+  { "enable-logging",           no_argument, NULL, 0 },
+  { "no-enable-logging",        no_argument, NULL, 0 },
   {0, 0, 0, 0}
 };
 
@@ -382,22 +384,28 @@ void cmdline_opts_free(const kvz_api *const api, cmdline_opts_t *opts)
 
 void print_usage(void)
 {
+  print_version();
   fprintf(stdout,
-    "Kvazaar usage: -i and --input-res to set input, -o to set output\n"
-    "               --help for more information\n");
+    "usage: -i and --input-res to set input, -o to set output\n"
+    "              --help for more information\n");
 }
 
 
 void print_version(void)
 {
   fprintf(stdout,
-    "Kvazaar " VERSION_STRING "\n"
+#ifdef CMAKE_BUILD
+    "kvazaar " VERSION_STRING " [" KVZ_COMPILER_STRING "] " KVZ_COMPILE_DATE "\n");
+#else
+  "Kvazaar " VERSION_STRING "\n"
     "Kvazaar license: 3-clause BSD\n");
+#endif
 }
 
 
 void print_help(void)
 {
+  print_version();
   fprintf(stdout,
     "Usage:\n"
     "kvazaar -i <input> --input-res <width>x<height> -o <output>\n"
@@ -447,6 +455,8 @@ void print_help(void)
     "                                   - md5: 56 bytes\n"
     "      --(no-)psnr            : Calculate PSNR for frames. [enabled]\n"
     "      --(no-)info            : Add encoder info SEI. [enabled]\n"
+    "      --(no-)enable-logging  : Enable logging for regular encoder performance,\n"
+    "                               error messages are always disblayed. [enabled]\n"
     "      --crypto <string>      : Selective encryption. Crypto support must be\n"
     "                               enabled at compile-time. Can be 'on' or 'off' or\n"
     "                               a list of features separated with a '+'. [off]\n"
