@@ -100,6 +100,10 @@ kvz_picture * kvz_image_alloc(enum kvz_chroma_format chroma_format, const int32_
 
   im->interlacing = KVZ_INTERLACING_NONE;
 
+  im->roi.roi_array = NULL;
+  im->roi.width = 0;
+  im->roi.height = 0;
+
   return im;
 }
 
@@ -126,6 +130,7 @@ void kvz_image_free(kvz_picture *const im)
     kvz_image_free(im->base_image);
   } else {
     free(im->fulldata_buf);
+    if (im->roi.roi_array) FREE_POINTER(im->roi.roi_array);
   }
 
   // Make sure freed data won't be used.
@@ -186,6 +191,8 @@ kvz_picture *kvz_image_make_subimage(kvz_picture *const orig_image,
   im->pts = 0;
   im->dts = 0;
 
+  im->roi = orig_image->roi;
+
   return im;
 }
 
@@ -218,27 +225,6 @@ void kvz_yuv_t_free(yuv_t *yuv)
   FREE_POINTER(yuv);
 }
 
-hi_prec_buf_t * kvz_hi_prec_buf_t_alloc(int luma_size)
-{
-  // Get buffers with separate mallocs in order to take advantage of
-  // automatic buffer overrun checks.
-  hi_prec_buf_t *yuv = (hi_prec_buf_t *)malloc(sizeof(*yuv));
-  yuv->y = (int16_t *)malloc(luma_size * sizeof(*yuv->y));
-  yuv->u = (int16_t *)malloc(luma_size / 2 * sizeof(*yuv->u));
-  yuv->v = (int16_t *)malloc(luma_size / 2 * sizeof(*yuv->v));
-  yuv->size = luma_size;
-
-  return yuv;
-}
-
-void kvz_hi_prec_buf_t_free(hi_prec_buf_t * yuv)
-{
-  free(yuv->y);
-  free(yuv->u);
-  free(yuv->v);
-  free(yuv);
-}
-
 static INLINE uint32_t reg_sad_maybe_optimized(const kvz_pixel * const data1, const kvz_pixel * const data2,
                                   const int32_t width, const int32_t height, const uint32_t stride1,
                                   const uint32_t stride2, optimized_sad_func_ptr_t optimized_sad)
